# -*- coding: utf-8 -*-
def colorContext(context):
    if context == "Left":
        colorContext = "red"
    elif context == "Right":
        colorContext = "blue"
    else:
        colorContext = "black"
    return colorContext
